@@ -537,10 +537,9 @@ public:
   class Iterator;
   using value_type= T;
   using iterator= Iterator;
-  using const_iterator= const Iterator;
 
-  Iterator begin() const { return Iterator(first); }
-  Iterator end() const { return Iterator(); }
+  iterator begin() const { return iterator(first); }
+  iterator end() const { return iterator(); }
 
   class Iterator
   {
@@ -561,7 +560,7 @@ public:
       return *this;
     }
 
-    T operator++(int)
+    Iterator operator++(int)
     {
       Iterator tmp(*this);
       operator++();
@@ -677,7 +676,8 @@ struct ilink
   struct ilink **prev,*next;
   static void *operator new(size_t size) throw ()
   {
-    return (void*)my_malloc((uint)size, MYF(MY_WME | MY_FAE | ME_FATAL));
+    return (void*)my_malloc(PSI_INSTRUMENT_ME,
+                            (uint)size, MYF(MY_WME | MY_FAE | ME_FATAL));
   }
   static void operator delete(void* ptr_arg, size_t)
   {
